@@ -1,15 +1,20 @@
 #![allow(dead_code)]
 #![warn(clippy::perf)]
 #![feature(decl_macro)]
+#![feature(const_mut_refs)]
 #![feature(unchecked_math)]
 #![warn(clippy::complexity)]
 #![feature(core_intrinsics)]
 #![warn(clippy::correctness)]
 #![allow(non_camel_case_types)]
+#![feature(downcast_unchecked)]
 #![allow(non_upper_case_globals)]
 #![feature(destructuring_assignment)]
+#![feature(const_slice_from_raw_parts)]
 #![feature(const_fn_floating_point_arithmetic)]
 
+macro size_of { ($type:ty) => { std::mem::size_of::<$type>() } }
+#[inline] const fn alloc_len(size: usize) -> usize { return alloc_align(size).size(); }
 #[inline(always)] const unsafe fn unreachable() -> ! { std::hint::unreachable_unchecked(); }
 #[inline] fn alloc(size: usize) -> *mut u8 { unsafe { return std::alloc::alloc(alloc_align(size)); } }
 #[inline] fn free(ptr: *mut u8, size: usize) { unsafe { std::alloc::dealloc(ptr, alloc_align(size)); } }
@@ -27,8 +32,28 @@ pub struct framebuffer {
 unsafe impl Send for framebuffer {}
 unsafe impl Sync for framebuffer {}
 
+pub trait framebuffer_from<T> {
+  fn from(width: usize, height: usize, t: T) -> fb;
+}
+
 impl Drop for framebuffer {
-  fn drop(&mut self) { if self.ptr.0 { free(self.ptr_mut(), self.len()); } }
+  fn drop(&mut self) { if self.ptr.0 { free(self.as_mut_ptr(), self.len()); } }
+}
+
+impl std::ops::Deref for framebuffer {
+  type Target = [u8]; fn deref(&self) -> &Self::Target { return unsafe { std::slice::from_raw_parts(self.as_ptr(), self.len()) }; }
+}
+
+impl std::ops::DerefMut for framebuffer {
+  fn deref_mut(&mut self) -> &mut Self::Target { return unsafe { std::slice::from_raw_parts_mut(self.as_mut_ptr(), self.len()) }; }
+}
+
+impl std::ops::IndexMut<(usize, usize)> for framebuffer {
+  fn index_mut(&mut self, index: (usize, usize)) -> &mut Self::Output { return unsafe { &mut *(self.as_mut_ptr::<u32>().add(index.0 + index.1 * self.width) as *mut colors::rgba) }; }
+}
+
+impl std::ops::Index<(usize, usize)> for framebuffer {
+  type Output = colors::rgba; fn index(&self, index: (usize, usize)) -> &Self::Output { return unsafe { &*(self.as_ptr::<u32>().add(index.0 + index.1 * self.width) as *mut colors::rgba) }; }
 }
 
 impl Clone for framebuffer {
@@ -39,50 +64,41 @@ impl Clone for framebuffer {
   }
 }
 
+
+impl<T> framebuffer_from<*mut T> for *mut T { fn from(width: usize, height: usize, container: *mut T) -> fb { return fb { width, height, ptr: (false, container as *mut u8) }; } }
+impl<T> framebuffer_from<&[T]> for &[T] { fn from(width: usize, height: usize, container: &[T]) -> fb { return fb { width, height, ptr: (false, container.as_ptr() as *mut u8) }; } }
+impl<T> framebuffer_from<*const T> for *const T { fn from(width: usize, height: usize, container: *const T) -> fb { return fb { width, height, ptr: (false, container as *mut u8) }; } }
+impl<T> framebuffer_from<&Vec<T>> for &Vec<T> { fn from(width: usize, height: usize, container: &Vec<T>) -> fb { return fb { width, height, ptr: (false, container.as_ptr() as *mut u8) }; } }
+impl<T> framebuffer_from<&mut [T]> for &mut [T] { fn from(width: usize, height: usize, container: &mut [T]) -> fb { return fb { width, height, ptr: (false, container.as_mut_ptr() as *mut u8) }; } }
+impl<T> framebuffer_from<&mut Vec<T>> for &mut Vec<T> { fn from(width: usize, height: usize, container: &mut Vec<T>) -> fb { return fb { width, height, ptr: (false, container.as_mut_ptr() as *mut u8) }; } }
+impl framebuffer_from<colors::rgba> for colors::rgba { fn from(width: usize, height: usize, container: colors::rgba) -> fb { let mut fb = unsafe { framebuffer::new_uninit(width, height) }; ops::fill::color(&mut fb, container); return fb; } }
+impl framebuffer_from<colors::rgb> for colors::rgb { fn from(width: usize, height: usize, container: colors::rgb) -> fb { let mut fb = unsafe { framebuffer::new_uninit(width, height) }; ops::fill::color(&mut fb, container.into()); return fb; } }
+impl<F, T: ops::fill::fill_value> framebuffer_from<F> for F where F: Fn(usize, usize) -> T { fn from(width: usize, height: usize, container: F) -> fb { let mut fb = unsafe { framebuffer::new_uninit(width, height) }; ops::fill::function(&mut fb, container); return fb; } }
+
+
 impl framebuffer {
   pub fn new(width: usize, height: usize) -> Self { return Self { width, height, ptr: (true, calloc(4 * width * height)) }; }
-  pub const unsafe fn from_ptr(width: usize, height: usize, ptr: *mut u8) -> Self { return Self { width, height, ptr: (false, ptr) }; }
-  pub unsafe fn new_fast(width: usize, height: usize) -> Self { return Self { width, height, ptr: (true, alloc(4 * width * height)) }; }
-  pub const unsafe fn from(width: usize, height: usize, buffer: &[u8]) -> Self { return Self { width, height, ptr: (false, buffer.as_ptr() as _) }; }
-  pub unsafe fn from_mut(width: usize, height: usize, buffer: &mut [u8]) -> Self { return Self { width, height, ptr: (false, buffer.as_mut_ptr()) }; }
+  pub fn from<T: framebuffer_from<T>>(width: usize, height: usize, container: T) -> Self { return T::from(width, height, container); }
+  pub unsafe fn new_uninit(width: usize, height: usize) -> Self { return Self { width, height, ptr: (true, alloc(4 * width * height)) }; }
 
-  pub fn ptr_mut<T>(&mut self) -> *mut T { return self.ptr.1 as *mut T; }
-  pub const fn ptr<T>(&self) -> *const T { return self.ptr.1 as *const T; }
   pub const fn len(&self) -> usize { return 4 * self.width * self.height; }
-  pub fn slice<T>(&self) -> &[T] { return unsafe { std::slice::from_raw_parts(self.ptr(), self.len() / std::mem::size_of::<T>()) }; }
-  pub fn set(&mut self, x: usize, y: usize, color: colors::rgba) { unsafe { *self.ptr_mut::<u32>().add(x + y * self.width) = color.into() }; }
-  pub fn get(&self, x: usize, y: usize) -> colors::rgba { return unsafe { std::mem::transmute(*self.ptr::<u32>().add(x + y * self.width)) }; }
-  pub fn slice_mut<T>(&mut self) -> &mut [T] { return unsafe { std::slice::from_raw_parts_mut(self.ptr_mut(), self.len() / std::mem::size_of::<T>()) }; }
-  pub fn vec<T>(mut self) -> Vec<T> { self.ptr.0 = false; return unsafe { Vec::from_raw_parts(self.ptr.1 as *mut T, self.len() / std::mem::size_of::<T>(), alloc_align(self.len()).size() / std::mem::size_of::<T>()) }; }
+  pub const fn as_ptr<T>(&self) -> *const T { return self.ptr.1 as *const T; }
+  pub const fn as_mut_ptr<T>(&mut self) -> *mut T { return self.ptr.1 as *mut T; }
+  pub const fn as_slice<T>(&self) -> &[T] { return unsafe { std::slice::from_raw_parts(self.as_ptr(), self.len() / size_of!(T)) }; }
+  pub const fn as_mut_slice<T>(&mut self) -> &mut [T] { return unsafe { std::slice::from_raw_parts_mut(self.as_mut_ptr(), self.len() / size_of!(T)) }; }
+  pub fn into_vec<T>(mut self) -> Vec<T> { self.ptr.0 = false; return unsafe { Vec::from_raw_parts(self.as_mut_ptr(), self.len() / size_of!(T), alloc_len(self.len()) / size_of!(T)) }; }
 }
 
 pub mod colors {
   #[repr(C)] #[derive(Copy, Clone)] pub struct rgb { pub r: u8, pub g: u8, pub b: u8 }
   #[repr(C)] #[derive(Copy, Clone)] pub struct rgba { pub r: u8, pub g: u8, pub b: u8, pub a: u8 }
 
-  impl From<rgba> for rgb {
-    fn from(c: rgba) -> rgb { return rgb { r: c.r, g: c.g, b: c.b }; }
-  }
-
-  impl From<rgb> for rgba {
-    fn from(c: rgb) -> rgba { return rgba { r: c.r, g: c.g, b: c.b, a: 255 }; }
-  }
-
-  impl From<rgb> for u32 {
-    fn from(c: rgb) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | (255 << 24); }
-  }
-
-  impl From<rgba> for u32 {
-    fn from(c: rgba) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | ((c.a as u32) << 24); }
-  }
-
-  impl From<u32> for rgb {
-    fn from(c: u32) -> rgb { return rgb { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8 }; }
-  }
-
-  impl From<u32> for rgba {
-    fn from(c: u32) -> rgba { return rgba { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8, a: (c >> 24) as u8 }; }
-  }
+  impl From<rgba> for rgb { #[inline] fn from(c: rgba) -> rgb { return rgb { r: c.r, g: c.g, b: c.b }; } }
+  impl From<rgb> for rgba { #[inline] fn from(c: rgb) -> rgba { return rgba { r: c.r, g: c.g, b: c.b, a: 255 }; } }
+  impl From<rgb> for u32 { #[inline] fn from(c: rgb) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | (255 << 24); } }
+  impl From<rgba> for u32 { #[inline] fn from(c: rgba) -> u32 { return c.r as u32 | ((c.g as u32) << 8) | ((c.b as u32) << 16) | ((c.a as u32) << 24); } }
+  impl From<u32> for rgb { #[inline] fn from(c: u32) -> rgb { return rgb { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8 }; } }
+  impl From<u32> for rgba { #[inline] fn from(c: u32) -> rgba { return rgba { r: (c & 0xFF) as u8, g: ((c >> 8) & 0xFF) as u8, b: ((c >> 16) & 0xFF) as u8, a: (c >> 24) as u8 }; } }
 
   pub fn blend(bg: u32, fg: u32) -> u32 {
     let fa = fg >> 24;
@@ -103,7 +119,7 @@ pub mod ops {
 
     pub fn opacity(fb: &mut fb, mut amount: f32) {
       amount = amount.clamp(0.0, 1.0);
-      let u8 = unsafe { fb.ptr_mut::<u8>().offset(3) };
+      let u8 = unsafe { fb.as_mut_ptr::<u8>().offset(3) };
 
       let mut offset = 0;
       let len = fb.len();
@@ -116,7 +132,7 @@ pub mod ops {
     }
 
     pub fn brightness(fb: &mut fb, amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
 
       for o in 0..(fb.len() / 4) {
         unsafe {
@@ -133,7 +149,7 @@ pub mod ops {
     }
 
     pub fn contrast(fb: &mut fb, amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       let i = 255.0 * (0.5 - (0.5 * amount));
 
       for o in 0..(fb.len() / 4) {
@@ -151,7 +167,7 @@ pub mod ops {
     }
 
     pub fn saturate(fb: &mut fb, amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
 
       let filter: [f32; 9] = [
         0.213 + 0.787 * amount, 0.715 - 0.715 * amount, 0.072 - 0.072 * amount,
@@ -177,7 +193,7 @@ pub mod ops {
     }
 
     pub fn sepia(fb: &mut fb, mut amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       amount = (1.0 - amount).clamp(0.0, 1.0);
 
       let filter: [f32; 9] = [
@@ -204,7 +220,7 @@ pub mod ops {
     }
 
     pub fn grayscale(fb: &mut fb, mut amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       amount = (1.0 - amount).clamp(0.0, 1.0);
 
       let filter: [f32; 9] = [
@@ -231,7 +247,7 @@ pub mod ops {
     }
 
     pub fn hue_rotate(fb: &mut fb, deg: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       let cos = f32::cos(deg.to_radians());
       let sin = f32::sin(deg.to_radians());
 
@@ -243,7 +259,9 @@ pub mod ops {
 
       for o in 0..(fb.len() / 4) {
         unsafe {
-          use std::intrinsics::{fadd_fast as fa, fmul_fast as fm, float_to_int_unchecked as fi};
+          use std::intrinsics::{fadd_fast as fa, fmul_fast as fm};
+          #[cfg(any(target_arch = "x86", target_arch = "x86_64"))] const fn fi<_f, _i>(x: f32) -> u32 { return x as u32; }
+          #[cfg(not(any(target_arch = "x86", target_arch = "x86_64")))] use std::intrinsics::{float_to_int_unchecked as fi};
 
           let c = *u32.add(o);
           let rr = (c & 0xff) as f32;
@@ -260,7 +278,7 @@ pub mod ops {
 
     pub fn drop_shadow(fb: &mut fb, x: isize, y: isize, sigma: f32, color: Option<colors::rgba>) {
       let mut old = fb.clone();
-      let u32 = old.ptr_mut::<u32>();
+      let u32 = old.as_mut_ptr::<u32>();
       ops::blur::gaussian(&mut old, sigma);
 
       if color.is_some() {
@@ -283,7 +301,7 @@ pub mod ops {
     }
 
     pub fn invert(fb: &mut fb, mut amount: f32) {
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       amount = amount.clamp(0.0, 1.0);
 
       if 1.0 == amount {
@@ -335,14 +353,14 @@ pub mod ops {
     pub fn horizontal(fb: &mut fb) {
       let width = fb.width;
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       for y in 0..height { unsafe { std::slice::from_raw_parts_mut(u32.add(y * width), width).reverse(); } }
     }
 
     pub fn vertical(fb: &mut fb) {
       let width = fb.width;
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
 
       for y in 0..(height / 2) {
         let yoffset = y * width;
@@ -355,24 +373,29 @@ pub mod ops {
   pub mod fill {
     use super::*;
 
-    pub fn function<F>(fb: &mut fb, f: F) where F: Fn(usize, usize) -> u32 {
+    pub trait fill_value { fn value(self) -> u32; }
+    impl fill_value for u32 { fn value(self) -> u32 { return self; } }
+    impl fill_value for colors::rgb { fn value(self) -> u32 { return self.into(); } }
+    impl fill_value for colors::rgba { fn value(self) -> u32 { return self.into(); } }
+
+    pub fn function<F, T: fill_value>(fb: &mut fb, f: F) where F: Fn(usize, usize) -> T {
       let width = fb.width;
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
 
       for y in 0..height {
         let yoffset = y * width;
-        for x in 0..width { unsafe { *u32.add(x + yoffset) = f(x, y); } }
+        for x in 0..width { unsafe { *u32.add(x + yoffset) = f(x, y).value(); } }
       }
     }
 
     pub fn color(fb: &mut fb, color: colors::rgba) {
-      if color.r == color.g && color.r == color.b && color.r == color.a { unsafe { fb.ptr_mut::<u8>().write_bytes(color.r, fb.len()); } }
+      if color.r == color.g && color.r == color.b && color.r == color.a { unsafe { fb.as_mut_ptr::<u8>().write_bytes(color.r, fb.len()); } }
 
       else {
         let width = fb.width;
         let height = fb.height;
-        let u32 = fb.ptr_mut::<u32>();
+        let u32 = fb.as_mut_ptr::<u32>();
         unsafe { std::slice::from_raw_parts_mut(u32, width).fill(color.into()); }
         for y in 1..height { unsafe { u32.copy_to_nonoverlapping(u32.add(y * width), width); } }
       }
@@ -382,7 +405,7 @@ pub mod ops {
       let width = fb.width;
       let bg = color.into();
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
 
       for y in 0..height {
         let yoffset = y * width;
@@ -406,13 +429,13 @@ pub mod ops {
     use super::*;
 
     pub fn rotate180(fb: &mut fb) {
-      fb.slice_mut::<u32>().reverse();
+      fb.as_mut_slice::<u32>().reverse();
     }
 
     pub fn rotate90(fb: &mut fb) {
       let width = fb.width;
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       let o32 = alloc(fb.len()) as *mut u32;
       unsafe { u32.copy_to_nonoverlapping(o32, fb.len() / 4); }
 
@@ -432,7 +455,7 @@ pub mod ops {
     pub fn rotate270(fb: &mut fb) {
       let width = fb.width;
       let height = fb.height;
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       let o32 = alloc(fb.len()) as *mut u32;
       unsafe { u32.copy_to_nonoverlapping(o32, fb.len() / 4); }
 
@@ -452,8 +475,8 @@ pub mod ops {
     use super::*;
 
     pub fn replace(fb: &mut fb, fg: &fb, x: isize, y: isize) {
-      let f32 = fg.ptr::<u32>();
-      let b32 = fb.ptr_mut::<u32>();
+      let f32 = fg.as_ptr::<u32>();
+      let b32 = fb.as_mut_ptr::<u32>();
       let (bw, bh) = (fb.width as isize, fb.height as isize);
       let (fw, fh) = (fg.width as isize, fg.height as isize);
 
@@ -473,8 +496,8 @@ pub mod ops {
     }
 
     pub fn blend(fb: &mut fb, fg: &fb, x: isize, y: isize) {
-      let f32 = fg.ptr::<u32>();
-      let b32 = fb.ptr_mut::<u32>();
+      let f32 = fg.as_ptr::<u32>();
+      let b32 = fb.as_mut_ptr::<u32>();
       let (bw, bh) = (fb.width as isize, fb.height as isize);
       let (fw, fh) = (fg.width as isize, fg.height as isize);
 
@@ -514,8 +537,8 @@ pub mod ops {
     }
 
     pub fn background(fb: &mut fb, bg: &fb, x: isize, y: isize) {
-      let bb32 = bg.ptr::<u32>();
-      let b32 = fb.ptr_mut::<u32>();
+      let bb32 = bg.as_ptr::<u32>();
+      let b32 = fb.as_mut_ptr::<u32>();
       let (bw, bh) = (fb.width as isize, fb.height as isize);
       let (fw, fh) = (bg.width as isize, bg.height as isize);
 
@@ -578,10 +601,10 @@ pub mod ops {
     pub fn nearest(fb: &fb, width: usize, height: usize) -> fb {
       let owidth = fb.width;
       let oheight = fb.height;
-      let o32 = fb.ptr::<u32>();
-      let mut fb = framebuffer::new(width, height);
+      let o32 = fb.as_ptr::<u32>();
+      let mut fb = unsafe { framebuffer::new_uninit(width, height) };
 
-      let u32 = fb.ptr_mut::<u32>();
+      let u32 = fb.as_mut_ptr::<u32>();
       let xw = 1.0 / width as f64 * owidth as f64;
       let yw = 1.0 / height as f64 * oheight as f64;
 
@@ -597,11 +620,11 @@ pub mod ops {
     pub unsafe fn linear(fb: &fb, width: usize, height: usize) -> fb {
       let owidth = fb.width;
       let oheight = fb.height;
-      let o8 = fb.ptr::<u8>();
+      let o8 = fb.as_ptr::<u8>();
       let mut fb = framebuffer::new(width, height);
 
       let mut offset = 0;
-      let u8 = fb.ptr_mut::<u8>();
+      let u8 = fb.as_mut_ptr::<u8>();
       let width1 = 1.0 / (width - 1) as f64;
       let height1 = 1.0 / (height - 1) as f64;
 
@@ -639,11 +662,11 @@ pub mod ops {
     pub unsafe fn cubic(fb: &fb, width: usize, height: usize) -> fb {
       let owidth = fb.width;
       let oheight = fb.height;
-      let o8 = fb.ptr::<u8>();
+      let o8 = fb.as_ptr::<u8>();
       let mut fb = framebuffer::new(width, height);
 
       let mut offset = 0;
-      let u8 = fb.ptr_mut::<u8>();
+      let u8 = fb.as_mut_ptr::<u8>();
       let width1 = 1.0 / (width - 1) as f64;
       let height1 = 1.0 / (height - 1) as f64;
 
@@ -743,7 +766,7 @@ pub mod ops {
 
       unsafe {
         let o8 = alloc(fb.len());
-        let u8 = fb.ptr_mut::<u8>();
+        let u8 = fb.as_mut_ptr::<u8>();
         let f32 = alloc(4 * 4 * width.max(height)) as *mut f32;
 
         u8.copy_to(o8, fb.len());
